### Usage: 
### 1) to run the script without changing any headers: python set_sky-tplstart.py
### 2) to make changes: python set_sky-tplstart.py --doit

### Created 11.2018; F.P.A. Vogt; fvogt@eso.org | frederic.vogt@alumni.anu.edu.au 

### Last updated Oct 24, 2022 by Janette Suherli - jsuherli@gmail.com
### github.com/jsuherli

### A small python script to change the ESO TPL START value of 
### individual SKY fields to match that of OBJECT fields.

### !! Notes: place this script in the parent folder of the RAW data !!

# ============================

# Load the required packages
import os
import pickle
import argparse
from astropy.io import fits 

# Where is the data ?
data_loc = os.path.join('.','RAW')
pkl_fn = os.path.join('.','objsky_tplstart.pkl')

# Use argparse to have a "safety flag". Change the headers ONLY if the code is ran with --doit 
parser = argparse.ArgumentParser(description='Process some integers.')
parser.add_argument('--doit', action='store_true',
                          help='Actually change the FITS header keywors. No turning back!')
args = parser.parse_args()

# Warn the user about the danger ...
if not(args.doit):
    print('Not actually changing any header. Re-run with --doit once certain.')

# Ok, be careful here, because I will modify the data. Keep the original values of headers in a dedicated file - and never overwrite it
if not(os.path.isfile(pkl_fn)):

    # Get only the fits files
    fits_fn = [item for item in os.listdir(data_loc) if item[-3:]=='.fz']

    # Extract all the sky and object fields (slow since I need to check many calibs -? do both sky and objects together)
    objsky_fns = [item for item in fits_fn if fits.getval(os.path.join(data_loc,item),"HIERARCH ESO DPR TYPE") in ['OBJECT','SKY']]

    # Now get the SKY only
    sky_fns = [item for item in objsky_fns if fits.getval(os.path.join(data_loc,item),"HIERARCH ESO DPR TYPE")=="SKY"]
    obj_fns = [item for item in objsky_fns if fits.getval(os.path.join(data_loc,item),"HIERARCH ESO DPR TYPE")=="OBJECT"]

    # Sort these
    sky_fns.sort()
    obj_fns.sort()

    # Now, get the TPL START values for either
    sky_tplstart = [fits.getval(os.path.join(data_loc,item),"ESO TPL START") for item in sky_fns]
    obj_tplstart = [fits.getval(os.path.join(data_loc,item),"ESO TPL START") for item in obj_fns]

    # Now, combine both and sort it, to get them in the observation order
    tplstart = sky_tplstart + obj_tplstart
    tplstart.sort()

    # Now, save these to a pickle file, once and for all
    of  = open(pkl_fn,'wb')
    pickle.dump([sky_fns,obj_fns,sky_tplstart,tplstart],of)
    of.close()


# Ok, now, open the pkl file, and re-assign the different arrays (in case script is run multiple times
of = open(pkl_fn, 'rb')
[sky_fns, obj_fns, sky_tplstart, tplstart] = pickle.load(of)
of.close()

# Very well, I should now be able to assign a new TPL START to the SKY field, on th assumption that a given SKY is observed immediately after a given Science field
for (f,fn) in enumerate(sky_fns):
    
    if tplstart.index(sky_tplstart[f])+1 < len(tplstart):
        if sky_tplstart[f] == tplstart[tplstart.index(sky_tplstart[f])+1]:
            # My sky already has the same time as the Science ... don't do anything!
            print('%s: %s' % (fn, sky_tplstart[f]))

    else: 
        print('%s: %s -> %s' % (fn, sky_tplstart[f], tplstart[tplstart.index(sky_tplstart[f])-1]) )
    
        # Change the fits headers if requested by the user
        if args.doit:
            fits.setval(os.path.join(data_loc,fn),"ESO TPL START", value=tplstart[tplstart.index(sky_tplstart[f])-1])
